/*
 *  mdaOverdriveController.cpp
 *  mda-vst3
 *
 *  Created by Arne Scheffler on 6/14/08.
 *
 *  mda VST Plug-ins
 *
 *  Copyright (c) 2008 Paul Kellett
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include "mdaOverdriveController.h"

namespace Steinberg {
namespace Vst {
namespace mda {

	float OverdriveController::kParamDefDrive = 0.0f,
		OverdriveController::kParamDefDriveExpo = 0.0f,
		OverdriveController::kParamDefFilterCoeff = 0.84f,
		OverdriveController::kParamDefInput = 0.5f,
		OverdriveController::kParamDefMix = 1.0f,
		OverdriveController::kParamDefMuffle=0.0f,
		OverdriveController::kParamDefDriveMode = 0.0f,
		OverdriveController::kParamDefDriveModeCoeff = 0.10f,
		OverdriveController::kParamDefOutput = 0.5f,
	    OverdriveController::kParamDefAberration1 = 0.0f,
		OverdriveController::kParamDefAberration2 = 0.0f;

//-----------------------------------------------------------------------------
FUID OverdriveController::uid (0x12051428, 0x3400291E, 0x01002916, 0x05122AC4);

//-----------------------------------------------------------------------------
OverdriveController::OverdriveController ()
{
}

//-----------------------------------------------------------------------------
OverdriveController::~OverdriveController ()
{

}

//-----------------------------------------------------------------------------
tresult PLUGIN_API OverdriveController::initialize (FUnknown* context)
{
	tresult res = BaseController::initialize (context);
	if (res == kResultTrue)
	{
		paramsDefs = new ParamValue[kNumParams];
		paramsDefs[kParamDrive] = kParamDefDrive;
		paramsDefs[kParamDriveExpo] = kParamDefDriveExpo;
		paramsDefs[kParamFilterCoeff] = kParamDefFilterCoeff;
		paramsDefs[kParamInput] = kParamDefInput;
		paramsDefs[kParamMix] = kParamDefMix;
		paramsDefs[kParamMuffle] = kParamDefMuffle;
		paramsDefs[kParamDriveMode] = kParamDefDriveMode;
		paramsDefs[kParamDriveModeCoeff] = kParamDefDriveModeCoeff;
		paramsDefs[kParamOutput] = kParamDefOutput;
		paramsDefs[kParamAberration1] = kParamDefAberration1;
		paramsDefs[kParamAberration2] = kParamDefAberration2;

		parameters.addParameter (new ScaledParameter (USTRING("Drive"), USTRING("%"), 0, paramsDefs[kParamDrive], ParameterInfo::kCanAutomate, kParamDrive, 0, 100, true));
		parameters.addParameter (new ScaledParameter (USTRING("Muffle"), USTRING("%"), 0, paramsDefs[kParamMuffle], ParameterInfo::kCanAutomate, kParamMuffle, 0, 100, true));
		parameters.addParameter (new ScaledParameter (USTRING("Output"), USTRING("dB"), 0, paramsDefs[kParamOutput], ParameterInfo::kCanAutomate, kParamOutput, -20, 20, true));
		parameters.addParameter(new ScaledParameter(USTRING("FilterCoefficient"), USTRING(""), 0, paramsDefs[kParamFilterCoeff], ParameterInfo::kCanAutomate, kParamFilterCoeff, -10, 0,false));
		parameters.addParameter(new ScaledParameter(USTRING("Mix"), USTRING("%"), 0, paramsDefs[kParamMix], ParameterInfo::kCanAutomate, kParamMix, 0, 100, true));
		parameters.addParameter(new ScaledParameter(USTRING("Input"), USTRING("dB"), 0, paramsDefs[kParamInput], ParameterInfo::kCanAutomate, kParamInput, -20, 20, true));

		IndexedParameter* switchParam = new IndexedParameter(USTRING("Drive Expo"), 0, 1, paramsDefs[kParamDriveExpo], ParameterInfo::kCanAutomate | ParameterInfo::kIsList, kParamDriveExpo);
		switchParam->setIndexString(0, UString128("Lin"));
		switchParam->setIndexString(1, UString128("Expo"));
		parameters.addParameter(switchParam);

		switchParam = new IndexedParameter(USTRING("Drive Mode"), 0, 1, paramsDefs[kParamDriveMode], ParameterInfo::kCanAutomate | ParameterInfo::kIsList, kParamDriveMode);
		switchParam->setIndexString(0, UString128("Sqrt"));
		switchParam->setIndexString(1, UString128("SQRT!"));
		parameters.addParameter(switchParam);

		parameters.addParameter(new ScaledParameter(USTRING("DriveModeCoefficient"), USTRING(""), 0, paramsDefs[kParamDriveModeCoeff], ParameterInfo::kCanAutomate, kParamDriveModeCoeff, 0, 100, false));
	
		parameters.addParameter(new ScaledParameter(USTRING("Aberration +"), USTRING("%"), 0, paramsDefs[kParamAberration1], ParameterInfo::kCanAutomate, kParamAberration1, 0, 100, true));
		parameters.addParameter(new ScaledParameter(USTRING("Aberration -"), USTRING("%"), 0, paramsDefs[kParamAberration2], ParameterInfo::kCanAutomate, kParamAberration2, 0, 100, true));
	}
	return res;
}

//-----------------------------------------------------------------------------
tresult PLUGIN_API OverdriveController::terminate ()
{
	return BaseController::terminate ();
}

//-----------------------------------------------------------------------------
tresult PLUGIN_API OverdriveController::getParamStringByValue (ParamID tag, ParamValue valueNormalized, String128 string)
{
	return BaseController::getParamStringByValue (tag, valueNormalized, string);
	/*
	UString128 result;
		switch (tag)
		{
			default:
				return BaseController::getParamStringByValue (tag, valueNormalized, string);
		}
		result.copyTo (string, 128);
		return kResultTrue;*/
	
}

//-----------------------------------------------------------------------------
tresult PLUGIN_API OverdriveController::getParamValueByString (ParamID tag, TChar* string, ParamValue& valueNormalized)
{
	// TODO
	return BaseController::getParamValueByString (tag, string, valueNormalized);
	/*
	switch (tag)
		{
			default:
				return BaseController::getParamValueByString (tag, string, valueNormalized);
		}
		return kResultFalse;*/
	
}

}}} // namespaces
